/// <reference types="node" />
import { EventEmitter } from "events";
import { Queue } from "./queue";
import { Connection } from "./connection";
import { SchedulerOptions } from "../types/options";
import { Jobs } from "../types/jobs";
/**
 * ## Events
 * ```
 * scheduler.on("start", () => {
 *   console.log("scheduler started");
 * });
 * scheduler.on("end", () => {
 *   console.log("scheduler ended");
 * });
 *  scheduler.on("poll", () => {
 *    console.log("scheduler polling");
 *  });
 *  scheduler.on("master", state => {
 *    console.log("scheduler became master");
 *  });
 *  scheduler.on("error", error => {
 *    console.log(`scheduler error >> ${error}`);
 *  });
 *  scheduler.on("cleanStuckWorker", (workerName, errorPayload, delta) => {
 *    console.log(
 *      `failing ${workerName} (stuck for ${delta}s) and failing job ${errorPayload}`
 *    );
 *  });
 *  scheduler.on("workingTimestamp", timestamp => {
 *    console.log(`scheduler working timestamp ${timestamp}`);
 *  });
 *  scheduler.on("transferredJob", (timestamp, job) => {
 *    console.log(`scheduler enquing job ${timestamp} >> ${JSON.stringify(job)}`);
 *  });
 * ```
 */
export declare class Scheduler extends EventEmitter {
    options: SchedulerOptions;
    jobs: Jobs;
    name: string;
    master: boolean;
    running: boolean;
    private processing;
    queue: Queue;
    connection: Connection;
    private timer;
    constructor(options: any, jobs?: {});
    connect(): Promise<void>;
    start(): Promise<void>;
    end(): Promise<unknown>;
    poll(): any;
    private pollAgainLater;
    private masterKey;
    private tryForMaster;
    private releaseMasterLock;
    private nextDelayedTimestamp;
    private enqueueDelayedItemsForTimestamp;
    private nextItemForTimestamp;
    private transfer;
    private cleanupTimestamp;
    private checkStuckWorkers;
    forceCleanWorker(workerName: any, delta: any): Promise<void>;
}
