/// <reference types="node" />
import { EventEmitter } from "events";
import { Connection } from "./connection";
import { Queue } from "./queue";
import { WorkerOptions } from "../types/options";
import { Job } from "../types/job";
import { Jobs } from "../types/jobs";
/**
 * ## Events
 * ```
 * worker.on("start", () => {
 *   console.log("worker started");
 * });
 * worker.on("end", () => {
 *   console.log("worker ended");
 * });
 * worker.on("cleaning_worker", (worker, pid) => {
 *   console.log(`cleaning old worker ${worker}`);
 * });
 * worker.on("poll", queue => {
 *   console.log(`worker polling ${queue}`);
 * });
 * worker.on("ping", time => {
 *   console.log(`worker check in @ ${time}`);
 * });
 * worker.on("job", (queue, job) => {
 *   console.log(`working job ${queue} ${JSON.stringify(job)}`);
 * });
 * worker.on("reEnqueue", (queue, job, plugin) => {
 *   console.log(`reEnqueue job (${plugin}) ${queue} ${JSON.stringify(job)}`);
 * });
 * worker.on("success", (queue, job, result) => {
 *  console.log(`job success ${queue} ${JSON.stringify(job)} >> ${result}`);
 * });
 * worker.on("failure", (queue, job, failure) => {
 *   console.log(`job failure ${queue} ${JSON.stringify(job)} >> ${failure}`);
 * });
 * worker.on("error", (error, queue, job) => {
 *   console.log(`error ${queue} ${JSON.stringify(job)}  >> ${error}`);
 * });
 * worker.on("pause", () => {
 *   console.log("worker paused");
 * });
 * ```
 */
export declare class Worker extends EventEmitter {
    options: WorkerOptions;
    jobs: Jobs;
    started: boolean;
    name: string;
    queues: Array<string>;
    queue: string;
    private originalQueue;
    error: Error | null;
    result: any;
    ready: boolean;
    running: boolean;
    working: boolean;
    private pingTimer;
    job: Job;
    connection: Connection;
    queueObject: Queue;
    id: number;
    constructor(options: any, jobs?: {});
    connect(): Promise<void>;
    start(): Promise<void>;
    private init;
    end(): any;
    private poll;
    private perform;
    performInline(func: any, args: any): Promise<any>;
    private completeJob;
    private succeed;
    private fail;
    private pause;
    private workingOn;
    private track;
    private ping;
    private untrack;
    checkQueues(): Promise<void>;
    private failurePayload;
    private stringQueues;
}
